package pacman;

import game.Game;
import game.control.AIControl;
import game.control.MapNode;
import game.objects.Drawable;
import geometry.Circle;
import geometry.Point2D;
import geometry.Vector2D;

import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

public class Map implements Drawable {

    public static int WALL = 0;
    public static int WAY = 1;
    public static int PILL = 2;
    public static int POWER_PILL = 3;
    public static int PARTIAL_WALL = 4;
    public static int GHOST_SPAWN = 5;
    public static int PAC_MAN_SPAWN = 6;

    public static Map getInstance(String filename, Game game)
            throws IOException {

        Map map = new Map(game);
        BufferedReader reader = new BufferedReader(new FileReader(new File(game
                .getConfig().getMapDir(), filename)));

        String line = reader.readLine();
        int pos = line.indexOf(':');
        int width = Integer.parseInt(line.substring(5, pos));
        int height = Integer
                .parseInt(line.substring(pos + 1, line.length() - 1));

        map.mapCells = new int[width][height];

        for (int y = 0; y < height; ++y) {

            line = reader.readLine();

            for (int x = 0; x < width; ++x) {

                map.mapCells[x][y] = Integer.parseInt(line.substring(x, x + 1));
            }
        }

        reader.close();

        return map;
    }

    protected int[][] mapCells;
    protected Game game;

    private Map(Game game) {
        this.game = game;
    }

    public void createControlNetwork() {

        MapNode[][] nodes = new MapNode[mapCells.length][mapCells[0].length];

        
        for (int x = 0; x < mapCells.length; ++x) {
            for (int y = 0; y < mapCells[x].length; ++y) {

                int val = mapCells[x][y];

                if (val == WALL) {
                    nodes[x][y] = null;
                } else {
                    Point2D pos = PacManGame.MAPCANVAS_ORIGIN.add(new Vector2D(
                            (x + 0.5) * game.getConfig().getCellWidth(),
                            (y + 0.5) * game.getConfig().getCellHeight()));

                    if (val == PARTIAL_WALL) {
                        nodes[x][y] = new PartialWallNode(pos);
                    } else {
                        nodes[x][y] = new MapNode(pos);

                        if (val == GHOST_SPAWN) {

                            Ghost ghost = new Ghost(game, new Circle(pos, game
                                    .getConfig().getCellHeight() / 3),
                                    PacManGame.GHOSTS_SPEED);
                            AIControl aicontrol = new AIControl(game, ghost);
                            aicontrol.setTargetNode(nodes[x][y]);
                            ((PacManObjectManager) game.getObjectManager())
                                    .addGhost(ghost);

                        } else if (val == PAC_MAN_SPAWN) {

                            PacMan pacMan = new PacMan(game, new Circle(pos,
                                    game.getConfig().getCellHeight() / 3),
                                    PacManGame.PACMAN_SPEED);
                            PacManKeyboardControl control = new PacManKeyboardControl(
                                    game, pacMan);
                            control.setTargetNode(nodes[x][y]);
                            ((PacManObjectManager) game.getObjectManager())
                                    .addPlayer(pacMan);

                        } else if (val == PILL) {
                            Pille pill = new Pille(game, new Circle(pos, game
                                    .getConfig().getCellHeight() / 5));
                            nodes[x][y].addEntity(pill);
                            ((PacManObjectManager) game.getObjectManager())
                                    .addPill(pill);
                        } else if (val == POWER_PILL) {
                            PowerPill pill = new PowerPill(game, new Circle(
                                    pos, game.getConfig().getCellHeight() / 4));
                            nodes[x][y].addEntity(pill);
                            ((PacManObjectManager) game.getObjectManager())
                                    .addPill(pill);
                        }
                    }
                }
            }
        }

        createLinks(nodes);
    }

    private void createLinks(MapNode[][] nodes) {

        for (int x = 0; x < nodes.length; ++x) {
            for (int y = 0; y < nodes[x].length; ++y) {
                
            	if (nodes[x][y] != null) {
//                    if (x == 0 && nodes[nodes.length - 1][y] != null) {
//                        nodes[x][y].addNeighbour(nodes[nodes.length - 1][y]);
//                    }
//                    if (y == 0 && nodes[x][nodes[x].length - 1] != null) {
//                        nodes[x][y].addNeighbour(nodes[x][nodes.length - 1]);
//                    }
                    if (x + 1 < nodes.length && nodes[x + 1][y] != null) {
                        nodes[x][y].addNeighbour(nodes[x + 1][y]);
                    }
                    if (y + 1 < nodes[x].length && nodes[x][y + 1] != null) {
                        nodes[x][y].addNeighbour(nodes[x][y + 1]);
                    }
                }
            }
        }
    }

    @Override
    public void draw(Graphics g) {

        for (int x = 0; x < mapCells.length; ++x) {
            for (int y = 0; y < mapCells[x].length; ++y) {

                BufferedImage image;
                if (mapCells[x][y] == WALL) {
                    image = game.getContentManager().getImage("wall");
                } else if (mapCells[x][y] == PARTIAL_WALL) {
                    image = game.getContentManager().getImage("partial-wall");
                } else {
                    continue;
                    // image = game.getContentManager().getImage("way");
                }

                g.drawImage(image,
                        (int) (PacManGame.MAPCANVAS_ORIGIN.getX() + x
                                * game.getContentManager().getConfig()
                                        .getCellWidth()),
                        (int) (PacManGame.MAPCANVAS_ORIGIN.getY() + y
                                * game.getContentManager().getConfig()
                                        .getCellHeight()), null);
            }
        }
    }
}