package pacman;

import game.Game;
import game.GameConfig;
import game.GameUpdate;
import game.objects.ObjectManager;
import game.res.ContentManager;
import geometry.Point2D;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.IOException;

public class PacManGame extends Game {

	public static final double GHOSTS_SPEED = 35;
	public static final double PACMAN_SPEED = 42;
	public static final Point2D MAPCANVAS_ORIGIN = new Point2D(42, 35);

	private GameConfig config;
	private Map map;
	private PacManObjectManager objectManager;
	private ContentManager contentManager;

	public PacManGame() {
		try {
			config = GameConfig.getInstance("game.config");
		} catch (IOException e) {
			config = new GameConfig();
		}

		objectManager = new PacManObjectManager(this);
		contentManager = new ContentManager(this);

		init();
	}

	@Override
	public void draw(Graphics g) {

		Image bufferImage = new BufferedImage(config.getWindowWidth(),
				config.getWindowHeight(), BufferedImage.TYPE_INT_RGB);
		Graphics bufferGraphics = bufferImage.getGraphics();

		bufferGraphics.setColor(Color.BLACK);
		bufferGraphics.fillRect(0, 0, config.getWindowWidth(),
				config.getWindowHeight());

		map.draw(bufferGraphics);
		objectManager.draw(bufferGraphics);
		g.drawImage(bufferImage, 0, 0, null);
	}

	@Override
	public GameConfig getConfig() {
		return config;
	}

	@Override
	public ContentManager getContentManager() {
		return contentManager;
	}

	@Override
	public ObjectManager getObjectManager() {
		return objectManager;
	}

	private void init() {
		contentManager.loadImage("pacman", "pacman.png");
		contentManager.loadImage("wall", "wall.png");
		contentManager.loadImage("way", "mapcell.png");
		contentManager.loadImage("partial-wall", "wall_special.png");
		contentManager.loadImage("ghost", "monster.png");
		contentManager
				.loadImage("vulnerable-monster", "monster_vulnerable.png");
		contentManager.loadImage("pill", "pill.png");
		contentManager.loadImage("power-pill", "powerpill.png");
		try {
			map = Map.getInstance("level1.dat", this);
			map.createControlNetwork();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public boolean isPaused() {
		return false;
	}

	@Override
	public void setPaused() {
	}

	@Override
	public void update(GameUpdate update) {
		objectManager.update(update);
	}

}
